//#define DUBUG_MESSAGES

using System;
using Server;
using Server.Items;
using Server.Misc;
using Server.Network;
using Server.Mobiles;
using System.Collections;
using System.Collections.Generic;

namespace Server.Engines.XmlSpawner2
{
	public class XmlGainPotion : XmlAttachment
	{		
		private string m_PotionName;
		private double m_Multiplier = 0;	
		private double m_Duration = 0;		//mins
		private SkillName[] m_SkillList;
		
		[CommandProperty( AccessLevel.GameMaster )]
		public string PotionName{ get{ return m_PotionName; } set { m_PotionName = value; } }

		[CommandProperty( AccessLevel.GameMaster )]
		public double Multiplier{ get{ return m_Multiplier; } set { m_Multiplier = value; } }

		[CommandProperty( AccessLevel.GameMaster )]
		public double Duration{ get{ return m_Duration; } set { m_Duration = value; } }

		[CommandProperty( AccessLevel.GameMaster )]
		public SkillName[] SkillList{ get{ return m_SkillList; } set { m_SkillList = value; } }

		[Attachable]
		public XmlGainPotion()
		{
			Name = "gain potion";
			Multiplier = 1.5;
			SkillList = new SkillName[0];
			Duration = 60; 				//mins
		}

		[Attachable]
		public XmlGainPotion( string name, double multiplier, SkillName[] skilllist, double duration )
		{
			PotionName = name;
			Multiplier = multiplier;
			SkillList = skilllist;
			Duration = duration;			//mins
		}

		public double DoGain( Skill checkskill, double gc )
		{							
			if( SkillList.Length == 0 ) 	//all skills
			{
				gc *= Multiplier;
				#if(DUBUG_MESSAGES)
	            if( ((Mobile)AttachedTo).AccessLevel >= AccessLevel.Administrator )
					((Mobile)AttachedTo).SendMessage( "Skillgain applied {0}", checkskill.Name);	           
				#endif
			}
			else 							//specified skills
			{
				for ( int i = 0; i < SkillList.Length ; ++i )
				{
					if( checkskill == ( (Mobile)AttachedTo).Skills[SkillList[i]] )
					{
						gc *= Multiplier;
						#if(DUBUG_MESSAGES)
	        		    if( ((Mobile)AttachedTo).AccessLevel >= AccessLevel.Administrator )
							((Mobile)AttachedTo).SendMessage( "Skillgain applied {0}", checkskill.Name);
						#endif
					}
				}
			}
			return gc;
		}

		public void CheckTimeRemaining()
		{		
			TimeSpan Expires = ExpirationEnd - DateTime.Now;
			
			if( Expires > TimeSpan.Zero )
				((Mobile)AttachedTo).SendMessage( "Your {0} is active for {1}", PotionName, FormatTimeRemaining(Expires) );
			else 
				Delete();
		}
		
		public string FormatTimeRemaining( TimeSpan t )
		{
			return String.Format( "{0:D2}h:{1:D2}m:{2:D2}s", t.Hours % 24, t.Minutes % 60, t.Seconds % 60 );
		}

		public override void OnAttach()
		{
			base.OnAttach();

			if(AttachedTo is PlayerMobile)
			{
				Expiration = (TimeSpan.FromMinutes(Duration)); 	//initialise decay counter

				CheckTimeRemaining();
			}
			else
				Delete();
		}
		
		public override void OnDelete()
		{
			((Mobile)AttachedTo).SendMessage( "Your {0} has expired", PotionName);
		}

		public XmlGainPotion(ASerial serial) : base(serial)
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize(writer);

			writer.Write( (int) 1 ); //version

			//version 1 [skill lists]
			writer.Write((int)SkillList.Length);
			for ( int i = 0; i < SkillList.Length; ++i )
				writer.Write( (int)SkillList[i] );
			//version 0 [basic potion]
			writer.Write((string)PotionName);
			writer.Write((double)Multiplier);
			writer.Write((double)Duration);
		}

		public override void Deserialize(GenericReader reader)
		{
			base.Deserialize(reader);

			int version = reader.ReadInt();

			switch ( version )
			{
				case 1: //version 1 [skill lists]
				{
					int count = reader.ReadInt();
					SkillList = new SkillName[count];

					for ( int i = 0; i < count; ++i )
						SkillList[i] = (SkillName)reader.ReadInt();					

					goto case 0;
				}
				case 0: //version 0 [basic potion]
				{			
					PotionName = reader.ReadString();
					Multiplier = reader.ReadDouble();
					Duration = reader.ReadDouble();
				}
				break;
			}
		}		
	}
}
